@@ -14,6 +14,7 @@ Contains the Engine, Page, Unit, and Pagination classes.
 """
 
 from __future__ import with_statement
+import abc
 import codecs
 import os
 import re
@@ -21,16 +22,23 @@ import sys
 import warnings
 from datetime import datetime
 from functools import partial, reduce
+from traceback import format_exc
 
 from volt.config import CONFIG, Config
-from volt.exceptions import *
-from volt.utils import path_import
+from volt.exceptions import EmptyUnitsWarning
+from volt.utils import LoggableMixin, cachedproperty, path_import, write_file
 
 
+# required engine config values
+_REQUIRED_ENGINE_CONFIG = ('URL', 'CONTENT_DIR', 'PERMALINK',)
+
+# required engine config for paginations
+_REQUIRED_ENGINE_PAGINATIONS = ('PAGINATIONS', 'UNITS_PER_PAGINATION',)
+
 # regex objects for unit header and permalink processing
 _RE_DELIM = re.compile(r'^---$', re.MULTILINE)
 _RE_SPACES = re.compile(r'\s([A|a]n??)\s|_|\s+')
-_RE_PRUNE = re.compile(r'A-|An-|[\!"#\$%&\'\(\)\*\+\,\./:;<=>\?@\[\\\]\^`\{\|\}~]')
+_RE_PRUNE = re.compile(r'A-|An-|[^a-zA-Z0-9_-]')
 _RE_MULTIPLE = re.compile(r'-+')
 _RE_PERMALINK = re.compile(r'(.+?)/+(?!%)')
 
@@ -46,19 +54,14 @@ def chain_item_permalinks(items):
     and next items.
 
     """
-    try:
-        for idx, item in enumerate(items):
-            if idx != 0:
-                setattr(item, 'permalink_prev', items[idx-1].permalink)
-            if idx != len(items) - 1:
-                setattr(item, 'permalink_next', items[idx+1].permalink)
-    except AttributeError:
-        raise ContentError("%s '%s' neighbor(s) does not have a "
-                           "permalink attribute." % \
-                           (item.__class__.__name__.capitalize(), item.id))
+    for idx, item in enumerate(items):
+        if idx != 0:
+            setattr(item, 'permalink_prev', items[idx-1].permalink)
+        if idx != len(items) - 1:
+            setattr(item, 'permalink_next', items[idx+1].permalink)
 
 
-class Engine(object):
+class Engine(LoggableMixin):
 
     """Base Volt Engine class.
 
@@ -67,29 +70,33 @@ class Engine(object):
     itself, but provides convenient unit processing methods for the
     subclassing engine.
 
-    Any subclass of Engine must override these methods:
-    - activate
-    - dispatch
-    - create_units
+    Any subclass of Engine must create a 'units' property and override the
+    dispatch method. Optionally, the preprocess method may be overridden if any
+    unit processing prior to plugin run needs to be performed.
 
     """
 
+    __metaclass__ = abc.ABCMeta
+
     DEFAULTS = Config()
 
     def __init__(self):
-        self.units = list()
-        self.paginations = dict()
         self.config = Config(self.DEFAULTS)
+        self.logger.debug('created: %s' % type(self).__name__)
+        # attributes below are placeholders for template access later on
+        self.widgets = {}
 
-    def activate(self):
-        """Performs initial processing of resources into unit objects."""
-        raise NotImplementedError("%s must implement an activate method." % \
-                self.__class__.__name__)
+    def preprocess(self):
+        """Performs initial processing of units before plugins are run."""
+        pass
 
+    @abc.abstractmethod
     def dispatch(self):
         """Performs final processing after all plugins are run."""
-        raise NotImplementedError("%s must implement a dispatch method." % \
-                self.__class__.__name__)
+
+    @abc.abstractmethod
+    def units(self):
+        """Units of the engine."""
 
     def prime(self):
         """Consolidates default engine Config and user-defined Config.
@@ -100,40 +107,51 @@ class Engine(object):
         """
         # get user config object
         conf_name = os.path.splitext(os.path.basename(CONFIG.VOLT.USER_CONF))[0]
-        voltconf = path_import(conf_name, CONFIG.VOLT.ROOT_DIR)
+        user_conf = path_import(conf_name, CONFIG.VOLT.ROOT_DIR)
+
+        # custom engines must define an entry name for the user's voltconf
+        if not hasattr (self, 'USER_CONF_ENTRY'):
+            message = "%s must define a %s value as a class attribute." % \
+                    (type(self).__name__, 'USER_CONF_ENTRY')
+            self.logger.error(message)
+
+        # use default config if the user does not specify any
         try:
-            user_config = getattr(voltconf, self.USER_CONF_ENTRY)
+            user_config = getattr(user_conf, self.USER_CONF_ENTRY)
         except AttributeError:
-            raise ConfigError("%s must define a 'USER_CONF_ENTRY' value as a "
-                              "class attribute." % self.__class__.__name__)
+            user_config = Config()
 
         # to ensure proper Config consolidation
         if not isinstance(user_config, Config):
-            raise TypeError("User Config object '%s' must be a Config instance." % \
-                    self.USER_CONF_ENTRY)
+            message = "User Config object '%s' must be a Config instance." % \
+                    self.USER_CONF_ENTRY
+            self.logger.error(message)
+            raise TypeError(message)
+        else:
+            self.config.update(user_config)
 
-        self.config.update(user_config)
-
-        # set absolute directory paths
-        try:
-            self.config.CONTENT_DIR = os.path.join(CONFIG.VOLT.CONTENT_DIR, \
-                    self.config.CONTENT_DIR)
-        except AttributeError:
-            raise ConfigError("%s must define a 'CONTENT_DIR' value in "
-                              "DEFAULTS." % self.__class__.__name__)
+        # check attributes that must exist
+        for attr in _REQUIRED_ENGINE_CONFIG:
+            try:
+                getattr(self.config, attr)
+            except AttributeError:
+                message = "%s Config '%s' value is undefined." % \
+                        (type(self).__name__, attr)
+                self.logger.error(message)
+                self.logger.debug(format_exc())
+                raise
 
+        # set engine config paths to absolute paths
+        self.config.CONTENT_DIR = os.path.join(CONFIG.VOLT.CONTENT_DIR, \
+                self.config.CONTENT_DIR)
         for template in [x for x in self.config.keys() if x.endswith('_TEMPLATE')]:
                 self.config[template] = os.path.join(CONFIG.VOLT.TEMPLATE_DIR, \
                         self.config[template])
 
-    def create_units(self):
-        """Creates the units that will be processed by the engine."""
-        raise NotImplementedError("%s must implement a create_units method." % \
-                self.__class__.__name__)
-
     def chain_units(self):
         """Sets the previous and next permalink attributes of each unit."""
         chain_item_permalinks(self.units)
+        self.logger.debug('done: chaining units')
 
     def sort_units(self):
         """Sorts a list of units according to the given header field name."""
@@ -143,13 +161,18 @@ class Engine(object):
         try:
             self.units.sort(key=lambda x: getattr(x, sort_key), reverse=reversed)
         except AttributeError:
-            raise ContentError("Sorting key '%s' not present in all unit "
-                               "header field." % sort_key)
+            message = "Sort key '%s' not present in all units." % sort_key
+            self.logger.error(message)
+            self.logger.debug(format_exc())
+            raise
 
-    def create_paginations(self):
-        """Returns paginations of engine units in a dictionary.
+        self.logger.debug("done: sorting units based on '%s'" % self.config.SORT_KEY)
 
-        This method will expand the supplied patterns according to the values
+    @cachedproperty
+    def paginations(self):
+        """Paginations of engine units in a dictionary.
+
+        The computation will expand the supplied patterns according to the values
         present in all units. For example, if the pattern is '{time:%Y}' and
         there are five units with a datetime.year attribute 2010 and another
         five with 2011, create_paginations will return a dictionary with one key
@@ -158,32 +181,28 @@ class Engine(object):
         many units are in one pagination.
 
         """
-        try:
-            base_url = self.config.URL.strip('/')
-        except AttributeError:
-            raise ConfigError("%s Config must define a 'URL' value if "
-                              "create_paginations is used." % \
-                              self.__class__.__name__)
-        try:
-            units_per_pagination = self.config.UNITS_PER_PAGINATION
-        except AttributeError:
-            raise ConfigError("%s Config must define a 'UNITS_PER_PAGINATION' value "
-                              "if create_paginations is used." % \
-                              self.__class__.__name__)
-        try:
-            pagination_patterns = self.config.PAGINATIONS
-        except AttributeError:
-            raise ConfigError("%s Config must define a 'PAGINATIONS' value "
-                              "if create_paginations is used." % \
-                              self.__class__.__name__)
+        # check attributes that must exist
+        for attr in _REQUIRED_ENGINE_PAGINATIONS:
+            try:
+                getattr(self.config, attr)
+            except AttributeError:
+                message = "%s Config '%s' value is undefined." % \
+                        (type(self).__name__, attr)
+                self.logger.error(message)
+                self.logger.debug(format_exc())
+                raise
+
+        base_url = self.config.URL.strip('/')
+        units_per_pagination = self.config.UNITS_PER_PAGINATION
+        pagination_patterns = self.config.PAGINATIONS
 
         # create_paginations operates on self.units
         units = self.units
         if not units:
-            warnings.warn("%s has no units to paginate." % self.__class__.__name__, \
+            warnings.warn("%s has no units to paginate." % type(self).__name__, \
                     EmptyUnitsWarning)
             # exit function if there's no units to process
-            return
+            return {}
 
         paginator_map = {
                 'all': self._paginate_all,
@@ -195,7 +214,7 @@ class Engine(object):
                 'datetime': self._paginate_datetime,
         }
 
-        paginations = dict()
+        paginations = {}
         for pattern in pagination_patterns:
 
             perm_tokens = re.findall(_RE_PERMALINK, pattern.strip('/') + '/')
@@ -204,8 +223,9 @@ class Engine(object):
             # only the last token is allowed to be enclosed in '{}'
             for token in base_permalist[:-1]:
                 if '{%s}' % token[1:-1] == token:
-                    raise PermalinkTemplateError("Pagination pattern %s has "
-                            "non-last curly braces-enclosed field " % pattern)
+                    message = "Pagination pattern %s is invalid." % pattern
+                    self.logger.error(message)
+                    raise ValueError(message)
 
             # determine which paginate method to use based on field type
             last_token = base_permalist[-1]
@@ -218,29 +238,41 @@ class Engine(object):
 
             try:
                 paginate = paginator_map[field_type]
+            except KeyError:
+                message = "Pagination method for '%s' has not been " \
+                          "implemented." % field_type
+                self.logger.error(message)
+                self.logger.debug(format_exc())
+                raise
+            else:
                 args = [field, base_permalist, units_per_pagination]
-                paginate_list = [pagination.next() for pagination in paginate(*args)]
+                paginate_list = list(paginate(*args))
                 key = '/'.join(base_permalist)
                 paginations[key] = paginate_list
-            except KeyError:
-                raise NotImplementedError("Pagination method for '%s' has not "
-                                          "been implemented." % field_type)
 
         return paginations
 
     def _paginate_all(self, field, base_permalist, units_per_pagination):
         """Create paginations for all field values (PRIVATE)."""
-        yield self._paginator(self.units, base_permalist, units_per_pagination)
+        paginated = self._paginator(self.units, base_permalist, units_per_pagination)
+
+        self.logger.debug('created: %d %s paginations' % (len(paginated), 'all'))
+        return paginated
 
     def _paginate_single(self, field, base_permalist, units_per_pagination):
         """Create paginations for string/int/float header field values (PRIVATE)."""
         units = self.units
         str_set = set([getattr(x, field) for x in units])
 
+        paginated = []
         for item in str_set:
             matches = [x for x in units if item == getattr(x, field)]
             base_permalist = base_permalist[:-1] + [str(item)]
-            yield self._paginator(matches, base_permalist, units_per_pagination)
+            pagin = self._paginator(matches, base_permalist, units_per_pagination)
+            paginated.extend(pagin)
+
+        self.logger.debug('created: %d %s paginations' % (len(paginated), field))
+        return paginated
 
     def _paginate_multiple(self, field, base_permalist, units_per_pagination):
         """Create paginations for list or tuple header field values (PRIVATE)."""
@@ -248,10 +280,15 @@ class Engine(object):
         item_list_per_unit = (getattr(x, field) for x in units)
         item_set = reduce(set.union, [set(x) for x in item_list_per_unit])
 
+        paginated = []
         for item in item_set:
             matches = [x for x in units if item in getattr(x, field)]
             base_permalist = base_permalist[:-1] + [str(item)]
-            yield self._paginator(matches, base_permalist, units_per_pagination)
+            pagin = self._paginator(matches, base_permalist, units_per_pagination)
+            paginated.extend(pagin)
+
+        self.logger.debug('created: %d %s paginations' % (len(paginated), field))
+        return paginated
 
     def _paginate_datetime(self, field, base_permalist, units_per_pagination):
         """Create paginations for datetime header field values (PRIVATE)."""
@@ -267,6 +304,7 @@ class Engine(object):
         time_strs = [[x.strftime(y) for x in unit_times] for y in time_tokens]
         time_set = set(zip(*time_strs))
 
+        paginated = []
         # create placeholders for new tokens
         base_permalist = base_permalist[:-1] + [None] * len(time_tokens)
         for item in time_set:
@@ -281,7 +319,11 @@ class Engine(object):
                     matches.append(unit)
 
             base_permalist = base_permalist[:-(len(time_tokens))] + list(item)
-            yield self._paginator(matches, base_permalist, units_per_pagination)
+            pagin = self._paginator(matches, base_permalist, units_per_pagination)
+            paginated.extend(pagin)
+
+        self.logger.debug('created: %d %s paginations' % (len(paginated), field))
+        return paginated
 
     def _paginator(self, units, base_permalist, units_per_pagination):
         """Create paginations from units (PRIVATE).
@@ -292,7 +334,7 @@ class Engine(object):
         units_per_pagination -- Number of units to show per pagination.
 
         """
-        paginations = list()
+        paginations = []
 
         # count how many paginations we need
         is_last = len(units) % units_per_pagination != 0
@@ -310,19 +352,23 @@ class Engine(object):
             pagination = Pagination(units_in_pagination, idx, base_permalist)
             paginations.append(pagination)
 
-        chain_item_permalinks(paginations)
+        if len(paginations) > 1:
+            chain_item_permalinks(paginations)
+            self.logger.debug('done: chaining paginations')
 
-        for pagination in paginations:
-            yield pagination
+        return paginations
 
     def write_units(self):
         """Writes units using the unit template file."""
         self._write_items(self.units, self.config.UNIT_TEMPLATE)
+        self.logger.debug('written: %d %s unit(s)' % (len(self.units), \
+                type(self).__name__[:-len('Engine')]))
 
     def write_paginations(self):
         """Writes paginations using the pagination template file."""
-        for pagination in self.paginations.values():
-            self._write_items(pagination, self.config.PAGINATION_TEMPLATE)
+        for pattern in self.paginations:
+            self._write_items(self.paginations[pattern], self.config.PAGINATION_TEMPLATE)
+            self.logger.debug("written: '%s' pagination(s)" % pattern)
 
     def _write_items(self, items, template_path):
         """Writes Page objects using the given template file (PRIVATE).
@@ -335,98 +381,101 @@ class Engine(object):
         template_env = CONFIG.SITE.TEMPLATE_ENV
         template_file = os.path.basename(template_path)
         template = template_env.get_template(template_file)
-        config_context = CONFIG
 
         for item in items:
             # warn if files are overwritten
             # this indicates a duplicate post, which could result in
             # unexpected results
             if os.path.exists(item.path):
-                raise DuplicateOutputError("'%s' already exists." % item.path)
-            try:
-                os.makedirs(os.path.dirname(item.path))
-            except OSError:
-                pass
-            with open(item.path, 'w') as target:
-                rendered = template.render(page=item.__dict__, \
-                        CONFIG=config_context)
-                self._write_output(target, rendered)
-
-    def _write_output(self, file_obj, string):
-        """Writes string to the open file object."""
-        if sys.version_info[0] < 3:
-            string = string.encode('utf-8')
-        file_obj.write(string)
+                message = "File %s already exists. Make sure there are no "\
+                          "other entries leading to this file path." % item.path
+                self.logger.error(message)
+                raise IOError(message)
+            else:
+                rendered = template.render(page=item, CONFIG=CONFIG, \
+                        widgets=self.widgets)
+                if sys.version_info[0] < 3:
+                    rendered = rendered.encode('utf-8')
+                write_file(item.path, rendered)
 
 
-class Page(object):
+class Page(LoggableMixin):
 
     """Class representing resources that may have its own web page, such as
     a Unit or a Pagination."""
 
+    __metaclass__ = abc.ABCMeta
+
     def __repr__(self):
-        return '%s(%s)' % (self.__class__.__name__, self.id)
+        return '%s(%s)' % (type(self).__name__, self.id)
+
+    @abc.abstractproperty
+    def permalist(self):
+        """List of tokens used to construct permalink and path."""
+
+    @abc.abstractproperty
+    def id(self):
+        """Unique string that identifies the Page object."""
+
+    @cachedproperty
+    def path(self):
+        """Filesystem path to Page object file."""
+        base_path = [CONFIG.VOLT.SITE_DIR]
+        base_path.extend(self.permalist)
+
+        if CONFIG.SITE.INDEX_HTML_ONLY:
+            base_path.append('index.html')
+        else:
+            base_path[-1] += '.html'
+
+        return os.path.join(*base_path)
+
+    @cachedproperty
+    def permalink(self):
+        """Relative URL to the Page object."""
+        rel_url = ['']
+        rel_url.extend(filter(None, self.permalist))
+
+        if CONFIG.SITE.INDEX_HTML_ONLY:
+            rel_url[-1] += '/'
+        else:
+            rel_url[-1] += '.html'
+
+        return '/'.join(rel_url)
+
+    @cachedproperty
+    def permalink_abs(self):
+        """Absolute URL to the Page object."""
+        return '/'.join([CONFIG.SITE.URL, self.permalink[1:]]).strip('/')
 
     def slugify(self, string):
         """Returns a slugified version of the given string."""
         string = string.strip()
 
+        # perform user-defined character mapping
+        for target in CONFIG.SITE.SLUG_CHAR_MAP:
+            string = string.replace(target, CONFIG.SITE.SLUG_CHAR_MAP[target])
+
         # replace spaces, etc with dash
         string = re.sub(_RE_SPACES, '-', string)
 
-        # remove english articles, bad chars, and dashes in front and end
+        # remove english articles, and non-ascii characters
         string = re.sub(_RE_PRUNE, '', string)
 
-        # raise exception if there are non-ascii chars
-        try:
-            if sys.version_info[0] > 2:
-                assert all(ord(c) < 128 for c in string)
-            else:
-                string.decode('ascii')
-        except (UnicodeDecodeError, AssertionError):
-            raise ContentError("Slug in '%s' contains non-ascii characters." \
-                               % self.id)
-
         # slug should not begin or end with dash or contain multiple dashes
         string = re.sub(_RE_MULTIPLE, '-', string)
 
         # and finally, we string preceeding and succeeding dashes
         string = string.lower().strip('-')
 
-        # raise exception if slug results in an empty string
+        # error if slug is empty
         if not string:
-            raise ContentError("Slug for '%s' is an empty string." % self.id)
+            message = "Slug for '%s' is an empty string." % self.id
+            self.logger.error(message)
+            raise ValueError(message)
 
         return string
 
-    def get_path_and_permalink(self):
-        """Returns the permalink and absolute file path."""
-        assert hasattr(self, 'permalist'), \
-                "%s requires the 'permalist' attribute to be set first." % \
-                self.__class__.__name__
-
-        abs_url = CONFIG.SITE.URL
-        index_html_only = CONFIG.SITE.INDEX_HTML_ONLY
-        base_path = [CONFIG.VOLT.SITE_DIR]
-        rel_url = ['']
-
-        permalist = self.permalist
-        base_path.extend(permalist)
-        rel_url.extend(filter(None, permalist))
-
-        if index_html_only:
-            rel_url[-1] = rel_url[-1] + '/'
-            base_path.append('index.html')
-        else:
-            rel_url[-1] = rel_url[-1] + '.html'
-            base_path[-1] = base_path[-1] + '.html'
-
-        path = os.path.join(*base_path)
-        permalink = '/'.join(rel_url)
-        permalink_abs = '/'.join([abs_url] + rel_url[1:]).strip('/')
-
-        return path, permalink, permalink_abs
-
 
 class Unit(Page):
 
@@ -437,64 +486,40 @@ class Unit(Page):
 
     """
 
-    def __init__(self, id):
-        """Initializes a unit instance with the given ID string."""
-        self.id = id
+    __metaclass__ = abc.ABCMeta
 
-    @property
-    def fields(self):
-        return self.__dict__.keys()
+    def __init__(self, config):
+        """Initializes a unit instance.
 
-    # convenience methods
-    open_text = partial(codecs.open, encoding='utf-8')
-    as_datetime = datetime.strptime
-    get_display_time = datetime.strftime
+        id -- Unique string to identify the unit.
+        config -- Config object of the calling Engine.
 
-    def check_protected(self, field, prot):
-        """Checks if the given field can be set by the user or not.
-        
-        field -- String to check against the list containing protected fields.
-        prot -- Iterable returning string of protected fields.
+        Config objects are required to instantiate the Unit since some unit
+        configuration values depends on the calling Engine configuration
+        values.
 
         """
-        if field in prot:
-            raise ContentError("'%s' should not define the protected header "
-                               "field '%s'" % (self.id, field))
+        if not isinstance(config, Config):
+            message = "Units must be instantiated with their engine's " \
+                    "Config object."
+            self.logger.error(message)
+            raise TypeError(message)
+        self.config = config
 
-    def check_required(self, req):
-        """Checks if all the required header fields are present.
-
-        req -- Iterable returning string of required header fields.
-
-        """
-        for field in req:
-            if not hasattr(self, field):
-                raise ContentError("Required header field '%s' is missing in "
-                                   "'%s'." % (field, self.id))
-
-    def as_list(self, field, sep):
-        """Transforms a character-separated string field into a list.
-
-        fields -- String to transform into list.
-        sep -- String used to split fields into list.
-
-        """
-        return list(set(filter(None, field.strip().split(sep))))
-
-    def get_permalist(self, pattern, unit_base_url='/'):
+    @cachedproperty
+    def permalist(self):
         """Returns a list of strings which will be used to construct permalinks.
 
-        pattern -- String replacement pattern.
-        unit_base_url -- String of base URL of the engine, to be appended in
-                         front of each unit URL.
+        For the permalist to be constructed, the calling Engine must define a
+        'PERMALINK' string in its Config object.
 
-        The pattern argument may refer to the current object's attributes by
-        enclosing them in square brackets. If the referred instance attribute 
+        The permalink string pattern may refer to the current unit's attributes
+        by enclosing them in square brackets. If the referred instance attribute
         is a datetime object, it must be formatted by specifying a string format
         argument.
 
         Several examples of a valid permalink pattern:
-        
+
         '{time:%Y/%m/%d}/{slug}'
             Returns, for example, ['2009', '10', '04', 'the-slug']
 
@@ -503,7 +528,8 @@ class Unit(Page):
 
         """
         # strip preceeding '/' but make sure ends with '/'
-        pattern = pattern.strip('/') + '/'
+        pattern = self.config.PERMALINK.strip('/') + '/'
+        unit_base_url = self.config.URL
 
         # get all permalink components and store into list
         perm_tokens = re.findall(_RE_PERMALINK, pattern)
@@ -515,19 +541,91 @@ class Unit(Page):
                 field = token[1:-1]
                 if ':' in field:
                     field, fmt = field.split(':')
-                if not hasattr(self, field):
-                    raise PermalinkTemplateError("'%s' has no '%s' attribute." % \
-                            (self.id, field))
-                if isinstance(getattr(self, field), datetime):
-                    strftime = datetime.strftime(getattr(self, field), fmt)
+
+                try:
+                    attr = getattr(self, field)
+                except AttributeError:
+                    message = "'%s' has no '%s' attribute." % (self.id, field)
+                    self.logger.error(message)
+                    self.logger.debug(format_exc())
+                    raise
+
+                if isinstance(attr, datetime):
+                    strftime = datetime.strftime(attr, fmt)
                     permalist.extend(filter(None, strftime.split('/')))
                 else:
-                    permalist.append(self.slugify(getattr(self, field)))
+                    permalist.append(self.slugify(attr))
             else:
                 permalist.append(self.slugify(token))
 
         return [unit_base_url.strip('/')] + filter(None, permalist)
 
+    # convenience methods
+    open_text = partial(codecs.open, encoding='utf-8')
+    as_datetime = datetime.strptime
+
+    def parse_header(self, header_string):
+        """Returns a dictionary of header field values.
+
+        header_string -- String of header lines.
+
+        """
+        header_lines = [x.strip() for x in header_string.strip().split('\n')]
+        for line in header_lines:
+            if not ':' in line:
+                    raise ValueError("Line '%s' in '%s' is not a proper "
+                            "header entry." % (line, self.id))
+            field, value = [x.strip() for x in line.split(':', 1)]
+
+            self.check_protected(field, self.config.PROTECTED)
+
+            if field == 'slug':
+                value = self.slugify(value)
+
+            elif field in self.config.FIELDS_AS_LIST:
+                value = self.as_list(value, self.config.LIST_SEP)
+
+            elif field in self.config.FIELDS_AS_DATETIME:
+                value = self.as_datetime(value, \
+                        self.config.DATETIME_FORMAT)
+
+            setattr(self, field.lower(), value)
+
+    def check_protected(self, field, prot):
+        """Checks if the given field can be set by the user or not.
+        
+        field -- String to check against the list containing protected fields.
+        prot -- Iterable returning string of protected fields.
+
+        """
+        if field in prot:
+            message = "'%s' should not define the protected header field " \
+                    "'%s'" % (self.id, field)
+            self.logger.error(message)
+            raise ValueError(message)
+
+    def check_required(self, req):
+        """Checks if all the required header fields are present.
+
+        req -- Iterable returning string of required header fields.
+
+        """
+        for field in req:
+            if not hasattr(self, field):
+                message = "Required header field '%s' is missing in '%s'." % \
+                        (field, self.id)
+                self.logger.error(message)
+                raise NameError(message)
+
+    def as_list(self, field, sep):
+        """Transforms a character-separated string field into a list.
+
+        fields -- String to transform into list.
+        sep -- String used to split fields into list.
+
+        """
+        return list(set(filter(None, field.strip().split(sep))))
+
 
 class Pagination(Page):
 
@@ -558,22 +656,19 @@ class Pagination(Page):
         self.pagin_idx = pagin_idx + 1
         # precautions for empty string, so double '/'s are not introduced
         self.base_permalist = filter(None, base_permalist)
+        self.logger.debug('created: %s' % self.id)
 
-        # set paths and permalist
-        self.permalist = self.get_permalist()
-        self.path, self.permalink, self.permalink_abs = self.get_path_and_permalink()
-        self.id = self.permalink
+    @cachedproperty
+    def id(self):
+        return self.permalink
 
-    def get_permalist(self):
+    @cachedproperty
+    def permalist(self):
         """Returns a list of strings which will be used to construct permalinks."""
-        pagination_url = CONFIG.SITE.PAGINATION_URL
-
-        if self.pagin_idx == 1:
-            # if it's the first pagination page, use base_permalist only
-            permalist = self.base_permalist
-        else:
-            # otherwise add pagination dir and pagination index
-            permalist = self.base_permalist + \
-                    filter(None, [pagination_url, str(self.pagin_idx)])
+        permalist = self.base_permalist
+        # add pagination url and index if it's not the first pagination page
+        if self.pagin_idx > 1:
+            permalist += filter(None, [CONFIG.SITE.PAGINATION_URL, \
+                    str(self.pagin_idx)])
 
         return permalist
