@@ -22,9 +22,9 @@ class TextileParserPlugin(Plugin):
 
     """Plugin for transforming textile syntax to html."""
 
-    def run(self, units):
-        """Process the given units."""
-        for unit in units:
+    def run(self, engine):
+        """Process the given engine."""
+        for unit in engine.units:
             if hasattr(unit, 'markup'):
                 is_textile = ('textile' == getattr(unit, 'markup').lower())
             else:
