@@ -23,23 +23,27 @@ to the parent class.
 
 """
 
-
+import logging
 import os
 import posixpath
 import sys
 import urllib
+from functools import partial
 from itertools import chain
 from socket import getfqdn
 from SimpleHTTPServer import SimpleHTTPRequestHandler
 from SocketServer import ThreadingTCPServer
 
-from volt import __version__
-from volt import gen
+from volt import VERSION
+from volt import generator
 from volt.config import CONFIG
-from volt.utils import notify, style
+from volt.utils import console, LoggableMixin
 
 
-class VoltHTTPServer(ThreadingTCPServer):
+console = partial(console, format="[srv] %s  %s\n")
+
+
+class VoltHTTPServer(ThreadingTCPServer, LoggableMixin):
 
     """A simple multithreading HTTP server for Volt development."""
 
@@ -59,6 +63,7 @@ class VoltHTTPServer(ThreadingTCPServer):
         """
         self.last_mtime = self.check_dirs_mtime()
         ThreadingTCPServer.__init__(self, *args, **kwargs)
+        self.logger.debug('created: %s' % type(self).__name__)
 
     def process_request(self, request, client_address):
         """Finishes one request by instantiating the handler class.
@@ -70,13 +75,17 @@ class VoltHTTPServer(ThreadingTCPServer):
 
         """
         latest_mtime = self.check_dirs_mtime()
+
         if self.last_mtime < latest_mtime:
+            message = "Source file modification detected -- regenerating site"
+            console(message, color='yellow')
+            self.logger.debug(message)
+
             self.last_mtime = latest_mtime
-            # generate the site
-            # not sure I understand why it needs to load first
-            # but hey it works (possible bug later on?)
-            CONFIG._load()
-            gen.run()
+            CONFIG.reset()
+            generator.run()
+            self.logger.debug('done: regenerating site')
+
         ThreadingTCPServer.process_request(self, request, client_address)
 
     def check_dirs_mtime(self):
@@ -94,7 +103,12 @@ class VoltHTTPServer(ThreadingTCPServer):
         # check the effects of changing certain configs
         dirs = (x[0] for x in os.walk(CONFIG.VOLT.ROOT_DIR) if
                 CONFIG.VOLT.SITE_DIR not in x[0] and CONFIG.VOLT.ROOT_DIR != x[0])
-        return max(os.stat(x).st_mtime for x in chain(dirs, [CONFIG.VOLT.USER_CONF]))
+
+        files = [CONFIG.VOLT.USER_CONF]
+        if os.path.exists(CONFIG.VOLT.USER_WIDGET):
+            files.append(CONFIG.VOLT.USER_WIDGET)
+
+        return max(os.stat(x).st_mtime for x in chain(dirs, files))
 
     def server_bind(self):
         # overrides server_bind to store the server name.
@@ -104,7 +118,7 @@ class VoltHTTPServer(ThreadingTCPServer):
         self.server_port = port
                                                     
 
-class VoltHTTPRequestHandler(SimpleHTTPRequestHandler):
+class VoltHTTPRequestHandler(SimpleHTTPRequestHandler, LoggableMixin):
 
     """HTTP request handler of the Volt HTTP server.
 
@@ -119,7 +133,7 @@ class VoltHTTPRequestHandler(SimpleHTTPRequestHandler):
 
     """
 
-    server_version = 'VoltHTTPServer/' + __version__
+    server_version = 'VoltHTTPServer/' + VERSION
 
     def log_error(self, format, *args):
         # overwritten to unclutter log message.
@@ -127,15 +141,16 @@ class VoltHTTPRequestHandler(SimpleHTTPRequestHandler):
 
     def log_message(self, format, *args):
         # overrides parent log_message to provide a more compact output.
-        message = "[%s] %s\n" % (self.log_date_time_string(), format % args)
+        message = format % args
 
         if int(args[1]) >= 400:
-            style(message, color='yellow')
+            console(message, color='red')
         elif int(args[1]) >= 300:
-            style(message, color='cyan')
+            console(message, color='cyan')
         else:
-            style(message)
-
+            console(message)
+        
+        self.logger.debug(message)
 
     def log_request(self, code='-', size='-'):
         # overrides parent log_request so 'size' can be set dynamically.
@@ -148,8 +163,10 @@ class VoltHTTPRequestHandler(SimpleHTTPRequestHandler):
                     size = os.path.getsize(actual_file)
             else:
                 size = os.path.getsize(self.file_path)
-        self.log_message('"%s" %s %s',
-                         self.requestline, str(code), str(size))
+
+        format = '"%s" %s %s'
+        args = (self.requestline[:-9], str(code), str(size))
+        self.log_message(format, *args)
 
     def translate_path(self, path):
         # overrides parent translate_path to enable custom directory setting.
@@ -174,6 +191,8 @@ def run():
     """Runs the HTTP server using options parsed by argparse, accessible
     via CONFIG.CMD."""
 
+    logger = logging.getLogger('server')
+
     address = ('127.0.0.1', CONFIG.CMD.server_port)
     try:
         server = VoltHTTPServer(address, VoltHTTPRequestHandler)
@@ -183,23 +202,36 @@ def run():
                       (CONFIG.CMD.server_port),
                   98: "Port %s already in use" % (CONFIG.CMD.server_port)}
         try:
-            error_message = ERRORS[e.args[0]]
+            message = ERRORS[e.args[0]]
         except (AttributeError, KeyError):
-            error_message = str(e)
-        style("Error: %s\n" % error_message, color='red', is_bright=True)
+            message = str(e)
+        logger.error(message)
         sys.exit(1)
 
     run_address, run_port = server.socket.getsockname()
     if run_address == '127.0.0.1':
         run_address = 'localhost'
-    style("\nVolt %s Development Server\n" % __version__, is_bright=True)
-    notify("Serving %s/\n" % CONFIG.VOLT.SITE_DIR, color='cyan')
-    notify("Running at http://%s:%s/\n"
-           "(CTRL-C to stop)\n\n" % (run_address, run_port), color='cyan')
+
+    message = "Volt %s Development Server" % VERSION
+    console(message, is_bright=True)
+    logger.debug(message)
+
+    message = "Serving %s" % CONFIG.VOLT.SITE_DIR
+    console(message)
+    logger.debug(message)
+
+    message = "Running at http://%s:%s" % (run_address, run_port)
+    console(message)
+    logger.debug(message)
 
     try:
         server.serve_forever()
-    except:
+    except KeyboardInterrupt:
         server.shutdown()
-        style("\nServer stopped.\n\n", is_bright=True)
+    finally:
+        sys.stdout.write("\n")
+        message = "Server stopped"
+        console(message)
+        logger.debug(message)
+        sys.stdout.write("\n")
         sys.exit(0)
