@@ -22,90 +22,35 @@ from mock import MagicMock, patch, call
 from volt.config import Config
 from volt.engine.core import Engine, Page, Unit, Pagination, \
         chain_item_permalinks
-from volt.exceptions import ConfigError, EmptyUnitsWarning, \
-        PermalinkTemplateError, ContentError
-from volt.test import USER_DIR
+from volt.exceptions import EmptyUnitsWarning
+from volt.test import USER_DIR, make_units_mock, make_uniconfig_mock
 
 
-def make_units_mock():
-    # define mock units
-    Unitlist_mock = list()
-    for i in range(5):
-        Unitlist_mock.append(MagicMock(spec=Unit))
-
-    # set unit attributes
-    unitlist_attrs = [
-            {'id': '1',
-             'title': 'Radical Notion',
-             'time': datetime(2010, 9, 5, 8, 0),
-             'tags': ['arthur', 'eames'],
-             'author': 'Smith',
-             },
-            {'id': '2',
-             'title': 'One Simple Idea',
-             'time': datetime(2010, 7, 30, 4, 31),
-             'author': 'Smith',
-             'tags': ['eames', 'arthur'],
-             'permalink': '/one',
-             },
-            {'id': '3',
-             'title': 'Dream Within A Dream',
-             'time': datetime(1998, 4, 5, 8, 0),
-             'author': 'Johnson',
-             'tags': ['eames', 'arthur', 'cobb'],
-             'permalink': '/dream',
-             },
-            {'id': '4',
-             'title': '528491',
-             'time': datetime(2002, 8, 17, 14, 35),
-             'author': 'Smith',
-             'tags': ['cobb', 'arthur'],
-             'permalink': '/528491',
-             },
-            {'id': '5',
-             'title': 'Dream is Collapsing',
-             'time': datetime(2011, 9, 5, 8, 0),
-             'tags': ['ariadne', 'arthur'],
-             'author': 'Johnson',
-             },
-            ]
-    for idx, attr in enumerate(unitlist_attrs):
-        for field in attr:
-            setattr(Unitlist_mock[idx], field, attr[field])
-
-    return Unitlist_mock
+UniConfig_mock = make_uniconfig_mock()
 
 
-def make_sessionconfig_mock():
+class TestEngine(Engine):
+    def dispatch(self): pass
+    @property
+    def units(self):
+        if not hasattr(self, '_lazy_units'):
+            setattr(self, '_lazy_units', make_units_mock())
+        return self._lazy_units
+    @units.setter
+    def units(self, units): self._lazy_units = units
 
-    configs = ['VOLT', 'SITE', ]
+class TestPage(Page):
+    @property
+    def id(self): return 'test'
+    @property
+    def permalist(self): return self._lazy_permalist
+    @permalist.setter
+    def permalist(self, permalist): self._lazy_permalist = permalist
 
-    VOLT = {'USER_CONF': os.path.join(USER_DIR, 'voltconf.py'),
-            'ROOT_DIR': USER_DIR,
-            'CONTENT_DIR': os.path.join(USER_DIR, 'content'),
-            'TEMPLATE_DIR': os.path.join(USER_DIR, 'templates'),
-            'SITE_DIR': os.path.join(USER_DIR, 'site'),
-           }
-    SITE = {'URL': 'http://foo.com',
-           }
-
-    sessionconfig_mock = MagicMock()
-
-    for config in configs:
-        setattr(sessionconfig_mock, config, MagicMock())
-        for key in eval(config):
-            setattr(getattr(sessionconfig_mock, config), key, eval(config)[key])
-
-    return sessionconfig_mock
-
-SessionConfig_mock = make_sessionconfig_mock()
+class TestUnit(Unit, TestPage): pass
 
 
-class TestEngineCoreMethods(unittest.TestCase):
-
-    def test_chain_item_permalinks_missing_neighbor_permalink(self):
-        units = make_units_mock()
-        self.assertRaises(ContentError, chain_item_permalinks, units)
+class EngineCoreMethodsCases(unittest.TestCase):
 
     def test_chain_item_permalinks_ok(self):
         units = make_units_mock()[1:-1]
@@ -122,27 +67,30 @@ class TestEngineCoreMethods(unittest.TestCase):
         self.assertEqual(units[-1].permalink_prev, units[1].permalink)
 
 
-@patch('volt.engine.core.CONFIG', SessionConfig_mock)
-class TestEngine(unittest.TestCase):
+@patch('volt.engine.core.CONFIG', UniConfig_mock)
+class EngineCases(unittest.TestCase):
 
     def setUp(self):
-        self.engine = Engine()
-
-    def test_create_units(self):
-        self.assertRaises(NotImplementedError, self.engine.create_units, )
-
-    def test_activate(self):
-        self.assertRaises(NotImplementedError, self.engine.activate, )
+        self.engine = TestEngine()
 
     def test_dispatch(self):
-        self.assertRaises(NotImplementedError, self.engine.dispatch, )
+        class TestEngine(Engine):
+            def activate(self): pass
+            def units(self): return
+        self.assertRaises(TypeError, TestEngine.__init__, )
+
+    def test_units(self):
+        class TestEngine(Engine):
+            def activate(self): pass
+            def dispatch(self): pass
+        self.assertRaises(TypeError, TestEngine.__init__, )
 
     def test_prime_user_conf_entry_none(self):
-        self.assertRaises(ConfigError, self.engine.prime, )
+        self.assertRaises(AttributeError, self.engine.prime, )
 
     def test_prime_content_dir_undefined(self):
         self.engine.USER_CONF_ENTRY = 'ENGINE_TEST'
-        self.assertRaises(ConfigError, self.engine.prime, )
+        self.assertRaises(AttributeError, self.engine.prime, )
 
     def test_prime_user_conf_not_config(self):
         self.engine.USER_CONF_ENTRY = 'ENGINE_TEST_BAD'
@@ -153,8 +101,10 @@ class TestEngine(unittest.TestCase):
         defaults = Config(
             BAR = 'engine bar in default',
             QUX = 'engine qux in default',
-            CONTENT_DIR = 'engine_test',
             UNIT_TEMPLATE = 'template.html',
+            URL = 'test',
+            CONTENT_DIR = 'engine_test',
+            PERMALINK = '',
         )
         self.engine.config = defaults
         self.engine.USER_CONF_ENTRY = 'ENGINE_TEST'
@@ -165,17 +115,15 @@ class TestEngine(unittest.TestCase):
         self.assertEqual(self.engine.config.BAR, 'engine bar in user')
         self.assertEqual(self.engine.config.QUX, 'engine qux in default')
         self.assertEqual(self.engine.config.CONTENT_DIR, os.path.join(\
-                USER_DIR, 'content', 'engine_test'))
+                USER_DIR, 'contents', 'engine_test'))
         self.assertEqual(self.engine.config.UNIT_TEMPLATE, os.path.join(\
                 USER_DIR, 'templates', 'template.html'))
 
     def test_sort_units_bad_key(self):
-        self.engine.units = make_units_mock()
         self.engine.config.SORT_KEY = 'date'
-        self.assertRaises(ContentError, self.engine.sort_units, )
+        self.assertRaises(AttributeError, self.engine.sort_units, )
 
     def test_sort_units_ok(self):
-        self.engine.units = make_units_mock()
         self.engine.config.SORT_KEY = '-time'
         titles = ['Dream is Collapsing', 'Radical Notion', 'One Simple Idea', \
                   '528491', 'Dream Within A Dream',]
@@ -183,69 +131,92 @@ class TestEngine(unittest.TestCase):
         self.engine.sort_units()
         self.assertEqual([x.title for x in self.engine.units], titles)
 
+    @patch('volt.engine.core.write_file')
+    def test_write_items_duplicate(self, write_mock):
+        template_path = 'item.html'
+        units = make_units_mock()[:2]
+        units[1].path = units[0].path
 
-class TestEnginePaginations(unittest.TestCase):
+        assert units[0].path == units[1].path
+        with open(units[1].path, 'w'):
+            self.assertRaises(IOError, self.engine._write_items, \
+                    units, template_path)
+        os.remove(units[1].path)
+
+    @patch('volt.engine.core.write_file')
+    def test_write_items_ok(self, write_mock):
+        template_path = 'item.html'
+        units = make_units_mock()[:2]
+        self.engine._write_items(units, template_path)
+
+        if sys.version_info[0] < 3:
+            rendered = '\xd1\x8e\xd0\xbd\xd0\xb8\xd0\xba\xd0\xbe\xd0\xb4\xd0\xb0'
+        else:
+            rendered = 'юникода'
+
+        call1 = call(os.path.join(USER_DIR, '1'), rendered + '|1')
+        call2 = call(os.path.join(USER_DIR, '2'), rendered + '|2')
+        self.assertEqual([call1, call2], write_mock.call_args_list)
+
+
+class EnginePaginationCases(unittest.TestCase):
 
     def setUp(self):
-        self.engine = Engine()
+        self.engine = TestEngine()
         self.engine.config.URL = 'test'
         self.engine.config.UNITS_PER_PAGINATION = 2
 
     def test_url_undefined(self):
         del self.engine.config.URL
         self.engine.config.PAGINATIONS = ('',)
-        self.assertRaises(ConfigError, self.engine.create_paginations, )
+        self.assertRaises(AttributeError, getattr, self.engine, 'paginations')
 
     def test_units_per_pagination_undefined(self):
         del self.engine.config.UNITS_PER_PAGINATION
         self.engine.config.PAGINATIONS = ('',)
-        self.assertRaises(ConfigError, self.engine.create_paginations, )
+        self.assertRaises(AttributeError, getattr, self.engine, 'paginations')
 
     def test_pagination_patterns_undefined(self):
-        self.assertRaises(ConfigError, self.engine.create_paginations, )
+        self.assertRaises(AttributeError, getattr, self.engine, 'paginations')
 
     @patch.object(warnings, 'warn')
     def test_empty_units_warning(self, warn_mock):
+        self.engine.units = []
         self.engine.config.PAGINATIONS = ('',)
-        self.engine.create_paginations()
-        args = [call('Engine has no units to paginate.', EmptyUnitsWarning)]
+        getattr(self.engine, 'paginations')
+        args = [call('TestEngine has no units to paginate.', EmptyUnitsWarning)]
         self.assertEqual(warn_mock.call_args_list, args)
 
     def test_bad_pagination_pattern(self):
-        self.engine.units = make_units_mock()
         self.engine.config.PAGINATIONS = ('{bad}/pattern',)
-        self.assertRaises(PermalinkTemplateError, \
-                self.engine.create_paginations, )
+        self.assertRaises(ValueError, getattr, self.engine, 'paginations')
 
     def test_paginate_not_implemented(self):
-        self.engine.units = make_units_mock()
         self.engine.config.PAGINATIONS = ('unimplemented/{newtype}',)
         for unit in self.engine.units:
             setattr(unit, 'newtype', dict(foo='bar'))
-        self.assertRaises(NotImplementedError, self.engine.create_paginations, )
+        self.assertRaises(KeyError, getattr, self.engine, 'paginations')
 
     @patch('volt.engine.core.Pagination')
-    def test_create_paginations_ok(self, Pagination_mock):
-        self.engine.units = make_units_mock()
+    def test_paginations_ok(self, Pagination_mock):
         pagination_patterns = ('',
                                'tag/{tags}',
                                'author/{author}',
                                '{time:%Y}',
                                '{time:%Y/%m}',)
-        expected = ['',
-                    'tag/arthur', 'tag/eames', 'tag/ariadne', 'tag/cobb',
-                    'author/Smith', 'author/Johnson',
+        expected = ['', '2', '3',
+                    'tag/arthur', 'tag/arthur/2', 'tag/arthur/3',
+                    'tag/eames', 'tag/eames/2', 'tag/ariadne', 'tag/cobb',
+                    'author/Smith', 'author/Smith/2', 'author/Johnson',
                     '2011', '2010', '2002', '1998',
                     '2011/09', '2010/09', '2010/07', '2002/08', '1998/04',]
 
         self.engine.config.PAGINATIONS = pagination_patterns
-        pagins = self.engine.create_paginations()
-        observed = sum([len(x) for x in pagins.values()])
+        observed = sum([len(x) for x in self.engine.paginations.values()])
         self.assertEqual(observed, len(expected))
 
     @patch('volt.engine.core.Engine._paginator')
     def test_paginate_all(self, paginator_mock):
-        self.engine.units = make_units_mock()
         base_permalist = ['test']
         field = base_permalist[-1][1:-1]
         [x for x in self.engine._paginate_all(field, base_permalist, 2)]
@@ -256,7 +227,6 @@ class TestEnginePaginations(unittest.TestCase):
 
     @patch('volt.engine.core.Engine._paginator')
     def test_paginate_single(self, paginator_mock):
-        self.engine.units = make_units_mock()
         base_permalist = ['test', 'author', '{author}']
         field = base_permalist[-1][1:-1]
         [x for x in self.engine._paginate_single(field, base_permalist, 2)]
@@ -270,7 +240,6 @@ class TestEnginePaginations(unittest.TestCase):
 
     @patch('volt.engine.core.Engine._paginator')
     def test_paginate_multiple(self, paginator_mock):
-        self.engine.units = make_units_mock()
         base_permalist = ['test', 'tag', '{tags}']
         field = base_permalist[-1][1:-1]
         [x for x in self.engine._paginate_multiple(field, base_permalist, 2)]
@@ -284,7 +253,6 @@ class TestEnginePaginations(unittest.TestCase):
 
     @patch('volt.engine.core.Engine._paginator')
     def test_paginate_datetime_single_time_token(self, paginator_mock):
-        self.engine.units = make_units_mock()
         base_permalist = ['test', '{time:%Y}']
         field = base_permalist[-1][1:-1]
         [x for x in self.engine._paginate_datetime(field, base_permalist, 2)]
@@ -298,7 +266,6 @@ class TestEnginePaginations(unittest.TestCase):
 
     @patch('volt.engine.core.Engine._paginator')
     def test_paginate_datetime_multiple_time_tokens(self, paginator_mock):
-        self.engine.units = make_units_mock()
         base_permalist = ['test', '{time:%Y/%m}']
         field = base_permalist[-1][1:-1]
         [x for x in self.engine._paginate_datetime(field, base_permalist, 2)]
@@ -314,7 +281,6 @@ class TestEnginePaginations(unittest.TestCase):
 
     @patch('volt.engine.core.Pagination')
     def test_paginator(self, Pagination_mock):
-        self.engine.units = make_units_mock()
         pagins = [p for p in self.engine._paginator(self.engine.units, ['base'], 2)]
 
         self.assertEqual(3, len(pagins))
@@ -324,28 +290,33 @@ class TestEnginePaginations(unittest.TestCase):
         Pagination_mock.assert_has_calls([call1, call2, call3], any_order=True)
 
 
-class TestPage(unittest.TestCase):
+class PageCases(unittest.TestCase):
 
     def setUp(self):
-        self.page = Page()
-        self.page.id = 'id'
+        self.page = TestPage()
 
     def test_repr(self):
         repr = self.page.__repr__()
-        self.assertEqual(repr, 'Page(id)')
+        self.assertEqual(repr, 'TestPage(test)')
 
-    def test_slugify_error(self):
+    @patch('volt.engine.core.CONFIG', MagicMock())
+    def test_slugify_empty(self):
         slugify = self.page.slugify
-        self.assertRaises(ContentError, slugify, 'Röyksopp - Eple')
-        self.assertRaises(ContentError, slugify, '宇多田ヒカル')
-        self.assertRaises(ContentError, slugify, '&**%&^%&$-')
+        cases = ['宇多田ヒカル', '&**%&^%&$-', u'ßÀœø']
+        for case in cases:
+            self.assertRaises(ValueError, slugify, case)
 
-    @patch.object(sys, 'version_info', [3])
-    def test_slugify_error_py3(self):
-        self.test_slugify_error()
+    @patch('volt.engine.core.CONFIG')
+    def test_slugify_char_map_ok(self, config_mock):
+        slugify = self.page.slugify
+        setattr(config_mock, 'SITE', Config())
+        config_mock.SITE.SLUG_CHAR_MAP = {u'ß': 'ss', u'ø': 'o'}
+        self.assertEqual(slugify(u'viel-spaß'), 'viel-spass')
+        self.assertEqual(slugify(u'Røyksopp'), 'royksopp')
+        self.assertEqual(slugify(u'ßnakeørama'), 'ssnakeorama')
 
+    @patch('volt.engine.core.CONFIG', MagicMock())
     def test_slugify_ok(self):
-        self.page.id = 'id'
         slugify = self.page.slugify
         self.assertEqual(slugify('Move along people, this is just a test'),
                 'move-along-people-this-is-just-test')
@@ -363,45 +334,44 @@ class TestPage(unittest.TestCase):
                 'journey-through-the-himalayan-mountains-part-1-unusual-guest')
 
     @patch('volt.engine.core.CONFIG.SITE.INDEX_HTML_ONLY', True)
-    @patch('volt.engine.core.CONFIG', SessionConfig_mock)
-    def test_get_path_and_permalink_index_html_true(self):
+    @patch('volt.engine.core.CONFIG', UniConfig_mock)
+    def test_path_permalinks_index_html_true(self):
         self.page.permalist = ['blog', 'not', 'string']
+        self.page.slugify = lambda x: x
 
-        path, permalink, permalink_abs = self.page.get_path_and_permalink()
-
-        self.assertEqual(path, os.path.join(USER_DIR, 'site', \
+        self.assertEqual(self.page.path, os.path.join(USER_DIR, 'site', \
                 'blog', 'not', 'string', 'index.html'))
-        self.assertEqual(permalink, '/blog/not/string/')
-        self.assertEqual(permalink_abs, 'http://foo.com/blog/not/string')
+        self.assertEqual(self.page.permalink, '/blog/not/string/')
+        self.assertEqual(self.page.permalink_abs, 'http://foo.com/blog/not/string')
 
     @patch('volt.engine.core.CONFIG.SITE.INDEX_HTML_ONLY', False)
-    @patch('volt.engine.core.CONFIG', SessionConfig_mock)
-    def test_get_path_and_permalink_index_html_false(self):
+    @patch('volt.engine.core.CONFIG', UniConfig_mock)
+    def test_path_permalinks_index_html_false(self):
         self.page.permalist = ['blog', 'not', 'string']
+        self.page.slugify = lambda x: x
 
-        path, permalink, permalink_abs = self.page.get_path_and_permalink()
-
-        self.assertEqual(path, os.path.join(USER_DIR, 'site', \
+        self.assertEqual(self.page.path, os.path.join(USER_DIR, 'site', \
                 'blog', 'not', 'string.html'))
-        self.assertEqual(permalink, '/blog/not/string.html')
-        self.assertEqual(permalink_abs, 'http://foo.com/blog/not/string.html')
+        self.assertEqual(self.page.permalink, '/blog/not/string.html')
+        self.assertEqual(self.page.permalink_abs, 'http://foo.com/blog/not/string.html')
 
 
-class TestUnit(unittest.TestCase):
+class UnitCases(unittest.TestCase):
 
     def setUp(self):
-        self.unit = Unit('01.md')
+        self.unit = TestUnit(Engine.DEFAULTS)
+        self.unit.config.URL = '/'
 
-    def test_fields(self):
-        self.assertEqual(self.unit.fields, ['id'])
+    def test_init(self):
+        self.assertRaises(TypeError, TestUnit.__init__, 'foo')
 
     def test_check_required(self):
         req = ('title', 'surprise', )
-        self.assertRaises(ContentError, self.unit.check_required, req)
+        self.assertRaises(NameError, self.unit.check_required, req)
 
     def test_check_protected(self):
         prot = ('cats', )
-        self.assertRaises(ContentError, self.unit.check_protected, 'cats', prot)
+        self.assertRaises(ValueError, self.unit.check_protected, 'cats', prot)
 
     def test_as_list_trailing(self):
         tags = 'ripley, ash, kane   '
@@ -418,53 +388,130 @@ class TestUnit(unittest.TestCase):
         taglist = ['trinity', 'twin', 'morpheus'].sort()
         self.assertEqual(self.unit.as_list(tags, ', ').sort(), taglist)
 
-    def test_get_permalist_error(self):
-        self.assertRaises(PermalinkTemplateError, self.unit.get_permalist, \
-                'bali/{beach}/party')
+    def test_permalist_missing_permalink(self):
+        self.unit.config.URL = '/'
+        del self.unit.config.PERMALINK
+        self.assertRaises(AttributeError, getattr, self.unit, 'permalist')
 
-    def test_get_permalist_ok(self):
-        get_permalist = self.unit.get_permalist
+    def test_permalist_missing_url(self):
+        self.unit.config.PERMALINK = 'foo'
+        del self.unit.config.URL
+        self.assertRaises(AttributeError, getattr, self.unit, 'permalist')
+
+    @patch('volt.engine.core.CONFIG', MagicMock())
+    def test_permalist_error(self):
+        self.unit.config.PERMALINK = 'bali/{beach}/party'
+        self.assertRaises(AttributeError, getattr, self.unit, 'permalist')
+
+    @patch('volt.engine.core.CONFIG', MagicMock())
+    def test_permalist_ok_all_token_is_attrib(self):
         self.unit.slug = 'yo-dawg'
         self.unit.time = datetime(2009, 1, 28, 16, 47)
-
-        self.assertEqual(get_permalist('{time:%Y/%m/%d}/{slug}'),
+        self.unit.config.PERMALINK = '{time:%Y/%m/%d}/{slug}'
+        self.assertEqual(self.unit.permalist, \
                 ['', '2009', '01', '28', 'yo-dawg'])
-        self.assertEqual(get_permalist('{time:%Y}/mustard/{time:%m}/{slug}/'),
+
+    @patch('volt.engine.core.CONFIG', MagicMock())
+    def test_permalist_ok_nonattrib_token(self):
+        self.unit.slug = 'yo-dawg'
+        self.unit.time = datetime(2009, 1, 28, 16, 47)
+        self.unit.config.PERMALINK = '{time:%Y}/mustard/{time:%m}/{slug}/'
+        self.assertEqual(self.unit.permalist, \
                 ['', '2009', 'mustard', '01', 'yo-dawg'])
-        self.assertEqual(get_permalist('i/love /mustard'),
+
+    @patch('volt.engine.core.CONFIG', MagicMock())
+    def test_permalist_ok_space_in_token(self):
+        self.unit.config.PERMALINK = 'i/love /mustard'
+        self.assertEqual(self.unit.permalist, \
                 ['', 'i', 'love', 'mustard'])
 
 
+class UnitHeaderCases(unittest.TestCase):
+
+    def setUp(self):
+        config = MagicMock(spec=Config)
+        config.GLOBAL_FIELDS = {}
+        TestUnit.title = 'a'
+        self.unit = TestUnit(config)
+
+    def test_parse_header_protected(self):
+        header_string = "content: this is a protected field"
+        self.unit.config.PROTECTED = ('content', )
+        self.assertRaises(ValueError, self.unit.parse_header, header_string)
+
+    @patch.object(TestUnit, 'slugify')
+    def test_parse_header_slug(self, slugify_mock):
+        slugify_mock.return_value = 'foo-slug'
+        header_string = "slug: foo-slug"
+        self.unit.parse_header(header_string)
+        self.assertEqual(self.unit.slug, 'foo-slug')
+
+    def test_parse_header_as_list(self):
+        self.unit.config.FIELDS_AS_LIST = 'tags'
+        self.unit.config.LIST_SEP = ', '
+        header_string = "tags: foo, bar, baz"
+        self.unit.parse_header(header_string)
+        expected = ['bar', 'baz', 'foo']
+        self.unit.tags.sort()
+        self.assertEqual(self.unit.tags, expected)
+
+    def test_parse_header_as_datetime(self):
+        self.unit.config.DATETIME_FORMAT = '%Y/%m/%d %H:%M'
+        self.unit.config.FIELDS_AS_DATETIME = ('time', )
+        header_string = "time: 2004/03/13 22:10"
+        self.unit.parse_header(header_string)
+        self.assertEqual(self.unit.time, datetime(2004, 3, 13, 22, 10))
+
+    def test_parse_header_extra_field(self):
+        header_string = "extra: surprise!"
+        self.unit.parse_header(header_string)
+        self.assertEqual(self.unit.extra, "surprise!")
+
+    def test_parse_header_empty_field(self):
+        header_string = "empty: "
+        self.unit.parse_header(header_string)
+        self.assertEqual(self.unit.empty, '')
+
+    def test_repr(self):
+        self.assertEqual(self.unit.__repr__(), 'TestUnit(test)')
+
+
 @patch('volt.engine.core.CONFIG.SITE.INDEX_HTML_ONLY', True)
-class TestPagination(unittest.TestCase):
+class PaginationCases(unittest.TestCase):
 
     def setUp(self):
         self.units = [MagicMock(Spec=Unit)] * 5
         self.site_dir = os.path.join(USER_DIR, 'site')
 
     @patch('volt.engine.core.CONFIG.SITE.PAGINATION_URL', 'page')
-    @patch('volt.engine.core.CONFIG', SessionConfig_mock)
+    @patch('volt.engine.core.CONFIG', UniConfig_mock)
+    def test_id(self):
+        pagin = Pagination(self.units, 0, )
+        self.assertEqual(pagin.id, '/')
+
+    @patch('volt.engine.core.CONFIG.SITE.PAGINATION_URL', 'page')
+    @patch('volt.engine.core.CONFIG', UniConfig_mock)
     def test_init_idx_0(self):
         pagin = Pagination(self.units, 0, )
         self.assertEqual(pagin.path, os.path.join(self.site_dir, 'index.html'))
         self.assertEqual(pagin.permalink, '/')
 
     @patch('volt.engine.core.CONFIG.SITE.PAGINATION_URL', 'page')
-    @patch('volt.engine.core.CONFIG', SessionConfig_mock)
+    @patch('volt.engine.core.CONFIG', UniConfig_mock)
     def test_init_idx_1(self):
         pagin = Pagination(self.units, 1, )
         self.assertEqual(pagin.path, os.path.join(self.site_dir, 'page', '2', 'index.html'))
         self.assertEqual(pagin.permalink, '/page/2/')
 
     @patch('volt.engine.core.CONFIG.SITE.PAGINATION_URL', 'page')
-    @patch('volt.engine.core.CONFIG', SessionConfig_mock)
+    @patch('volt.engine.core.CONFIG', UniConfig_mock)
     def test_init_permalist(self):
         pagin = Pagination(self.units, 1, ['tech'])
         self.assertEqual(pagin.path, os.path.join(self.site_dir, 'tech', 'page', '2', 'index.html'))
         self.assertEqual(pagin.permalink, '/tech/page/2/')
 
     @patch('volt.engine.core.CONFIG.SITE.PAGINATION_URL', '')
-    @patch('volt.engine.core.CONFIG', SessionConfig_mock)
+    @patch('volt.engine.core.CONFIG', UniConfig_mock)
     def test_init_pagination_url(self):
         pagin = Pagination(self.units, 1, )
         self.assertEqual(pagin.path, os.path.join(self.site_dir, '2', 'index.html'))
